{************************************
* ViewPort QuickSample         v2.0 *
* Adjustable 80 Mhz Sampling of INA *
* (C) 2008 myDanceBot.com, Inc.     *
*************************************}{{
Use this Object to take measurements of INA at very high speeds.
Users can use ViewPort to analyze those measurements alongside other variables
from their program.

Features:
-Measure 1440 samples at 32bit up to 80Mhz using 4 cogs.
-Measure 360 samples at 32bit up to 20Mhz using 1 cog.
-Flexible timescale from seconds to nanoseconds
-Edge and Pattern Triggers- can be reset by ViewPort if not Trigger found
-Samples are continuously taken by 1 or 4 interleaved cogs running self modifying assembly code.

Use:
-Include this object in your program:
        Obj qs:"QuickSample"
-Allocate memory for the INAFrame:
        LONG INAFrame[400]
        *Allocate ~400 longs for the INAFrame if using 1 cog, or 1600 if using 4.
-Register with ViewPort        
        vp.register(qs.sampleINA(@INAFrame,NumberOfCogs))
        *Use 1 cog to sample up to 20Mhz, or 4 to sample up to 80Mhz }}
CON
 _clkmode        = xtal1 + pll16x
 _xinfreq        = 5_000_000
  SamplesPCog      = 360
pub sampleINA(FrameAddr,nCogs):config 
 'frameAddr should point to long array of 406 elements:
 'tmask,tstate,tstate1,freq (0=20mhz, 16=5mhz, 32=2.5mhz...),syncA,data,syncB
 'syncA will be incremented when next measurement is started- syncB will be set to syncA when done
 'trigger active when mask<>0. does: waitpeq tstate, waitpeq tstate1- sample
 'cog may be restarted remotely from viewport 
  long[FrameAddr+16]:=cnt            '4=synca
  long[FrameAddr+12]:=nCogs-1        '3=freq
  long[FrameAddr]:=(@MeasureNoI << 2) | (frameAddr<<16)  
  cognew(@MeasureNoI,FrameAddr)
  if nCogs==4
    long[FrameAddr]:=long[FrameAddr]+1
    cognew(@MeasureNoI,FrameAddr)
    cognew(@MeasureNoI,FrameAddr)
    cognew(@MeasureNoI,FrameAddr)
  return FrameAddr | 2    
dat
              org 0
MeasureNoI    cogid     cogn
 'read from frame              
              mov       fptr,par 
              rdlong    tmask,fptr
              add       fptr,#4
              rdlong    tstate,fptr              
              add       fptr,#4
              rdlong    tstate1,fptr              
              add       fptr,#4
              rdlong    cps,fptr
              mov       ncog,cps  'cps=cps|2 bit ncog
              and       ncog,#3
              shr       cps,#2
              add       fptr,#4
              cmp       nsync,#0 wz 'on first time, nsync will be 0
    if_z      rdlong    nsync,fptr 'so load common sync from frame
    if_z      add       fptr,#4
    if_z      rdlong    xtime,fptr
 'only accept data if sa=sb.  sa=sb should guarantee all data belongs to 1 sample
 'sa,sb must surround data, so vp can make sure entire data is from 1 sample
 'ok for single, interleaved, 40mb modes: start reading sa- data filled in faster
 'datatime=samples*(40bit/long)/(2Mbit)*(80Mcycle/sec)=600k/2.4M (370/1480)
 'sampletime ~ 10k at full speed=insignificant
 'delay proportional to speed
 'old: wait cInt2, then write SA, sync before trigger, trigger, get nsync, sample, write data, write sB
 'new: sync, trigger, get sync, delay, sample, write SA, write data, write SB
 'only 1 sync

 'delay=min(cint,cint+delay-sTime) delay=ncog*xtime
 'on first round=cint delay,sTime=0
 'nsync+=delay- gets read
              add       nsync,cInt 'always add cInt to nsync
              add       nsync,delay 'sample time
              add       nsync,sTime 'send time
              mov       syncA,fptr
              mov       syncB,fptr
 'finc=4, set writeloop, set i, see how many cogs         
              mov       finc,#4          'finc=4
              mov       writeloop,wrC2F
              mov       i,#SamplesPCog
              cmp       ncog,#2 wc
              mov       delay,xTime
 'single cog: cwait=cps, fptr=data+4
 'cps=
 '0=20mhz in full mode, sample every 4cycles
 '<32=5mhz, sample every 16 cycles
 '>32=0..4mhz, sample every cts cycles
 '40=2mhz sample every 40cycles 80/40=2
    if_c      add       syncB,sAdd1
    if_c      mov       cWait,cps        'cWait=cps
    if_c      add       fptr,#4          'fptr=data+4
    if_c      jmp       #mkTrigger
 'four cog: clear z if trigger
 'cps=
 '+0=80mhz in 4*full mode, sample every 4cycles/4cogs=1cycle
 '2=40mhz in 2*(2*full) mode, sample every 4 cycles/2cogs=2cycles
 '4=20
 '8=10
 '16=5
 '<32=20mhz, sample every 16 cycles/4cog=4cycles
 '>32=0..20mhz, sample every cts cycles/4cogs=cts/4 cycles

 '     mov       cps,#2
              shl       delay,#2  'time to transmit 4xdata=4x
              add       syncB,sAdd4
              cmp       cps,#2    wc,wz
    if_z      jmp       #do40
 '80mhz or <=20Mhz: cWait 4+cts*cogn, fptr=data+4*cogn, finc=16
 '80mhz: cts=0 - cWait=4+cogn
 '40mhz
 '20mhz: cts=4 - cWait=4+4*cogn, between=16 because 16<32 so 16
              mov       finc,#16
              mov       cWait,#4
              mov       s1,#5
    calcWait  add       cWait,cps'
     if_c     add       cWait,#1 'only add 1 if cps=0
              add       fptr,#4 
              sub       s1,#1          
              djnz      cogn,#calcWait
              
    cWait2    djnz      s1,#cWait2
              tjz       cps,#mFull
              shl       cps,#2 'time to first is cps, between is 4*cps              
  mkTrigger   tjz       cps,#mFull1
              sub       cps,#17                 wc,nr
 'calculate sample time: 512*cps for 1 OR 4 cog  mode
              mov       stime,cps
              shl       stime,#9
              mov       measure,storeF
              waitcnt   nsync,0  'sync right before trigger
              tjz       tmask,#sslow 'skip next 2 if tmask=0
              waitpeq   tstate, tmask
              waitpeq   tstate1, tmask
sslow         mov       nSync,cnt      'nsync=cnt after trigger
              add       cWait,cnt
 'loop i times, wait if_nc, take sample, change destination
 'first sample taken 4 cycles, or cps after trigger, then every 4 or cps
 'requires cwait,cps, finc, fptr           
store  if_nc  waitcnt   cWait, cps
  measure     nop
              add       measure,d_inc
              djnz      i, #store
 'write cache to frame
writeFrame    mov       i, #SamplesPCog
              wrlong    nsync,syncA
writeloop     nop
              add       fptr, finc
              add       writeloop,d_inc
              djnz      i, #writeloop
 'write sync into end,  led off
              wrlong    nsync,syncB
 lk40         jmp       #MeasureNoI
              long      4      
              long      8
              long      SamplesPCog*8+12
              long      SamplesPCog*8+16
sAdd1         long      SamplesPCog*4+4
sAdd4         long      SamplesPCog*16+4
delay         long      0 'time to xmit all samples
stime         long      0 'time to sample
cInt          long      500_000 'constant wait
do40          '80mhz or <=20Mhz: cWait 4+cts*cogn, fptr=data+4*cogn, finc=16
 '80mhz: cts=0 - cWait=4+cogn
 '40mhz: finc=8, 
 '20mhz: cts=4 - cWait=4+4*cogn, between=16 because 16<32 so 16
              mov       finc,#8
              mov       doL,cogn  
              add       doL,doLook 'doL=mov s1,do40+1,2,3,4
              mov       cWait,#1 'need 1 instr between doL and mod to doL              
     doL      nop
              add       cWait,s1
              add       fptr,s1
              jmp       #mFull
 doLook       mov       s1,lk40  's1=contents of do40
syncA         long      0 
syncB         long      0 
nsync         Long      0 'time to start sampling, written at end                     
fPtr          Long      0 'points to frame
i             LONG      0 'variable
d_inc         LONG      $0000_0200 'increments destination in an opcode
ds_inc        LONG      $0000_0201 'increments destination/source in an opcode
cps           LONG      0 'cycles p sample to wait
cWait         Long      0 'wait counter
finc          Long      0 'amount to increment fptr during writing
xtime         long      0
 ' instructions to be written into code                      
jmpWF         jmp       #writeFrame
storeF        mov       fbufstart, ina               
wrc2f         wrlong    fbufstart, fptr
wrc2f1        wrlong    f1, fptr
dupRead       mov       s1,fbufstart          
modDup        add       s1,d_inc              
movJmp        mov       s1,jmpWF
f1w           mov       f1,ina
f2w           mov       f2,ina
tmask         nop 
tstate1       nop
tstate        nop
mFull1        mov       writeloop,wrC2F1
              mov       f1,f1w
              mov       f2,f2w
mFull         mov       :mk0,dupRead
              mov       :mk1,modDup
              mov       :mkjmp,movJmp
              mov       fbufstart,storeF
:mk0          nop
:mk1          nop
              add       :mk0,ds_inc           
              add       :mk1,d_inc            
              add       :mkjmp,d_inc          
              djnz      i,#:mk0
:mkjmp        nop
              waitcnt   nsync,0     'sync right before trigger
              tjz       tmask,#f0 'skip next 2 if tmask=0
              waitpeq   tstate, tmask
              waitpeq   tstate1, tmask
f0            mov       nsync,cnt      'nsync=cnt after trigger
f1            add       cWait,cnt
f2            waitcnt   cWait,#20
              
              fit       $1F0 - SamplesPCog         
fbufstart     nop  'will get overwritten with data, but reset at beginning
s1            nop
cogn          nop
ncog          nop